#!/bin/bash
# lliurex-netfuncs
# creado por Luis García para LliureX en Junio 2005


# variables

IFCONFIG="netcmd_wrapper /sbin/ifconfig"
ROUTE="netcmd_wrapper /sbin/route"
IPCALC="netcmd_wrapper /usr/bin/ipcalc"
ARP="netcmd_wrapper /usr/sbin/arp -n"
PING="netcmd_wrapper /bin/ping -c 1"
LSPCI="netcmd_wrapper /usr/bin/lspci"
LSUSB="netcmd_wrapper /usr/sbin/lsusb"
ETHTOOL="netcmd_wrapper /usr/sbin/ethtool"

NET_VAR_LIST="DEFAULT_SRV_IP DEFAULT_SRV_MASK SRV_NIC SRV_DOMAIN UNREG_HOSTNAME_PREFIX DEFAULT_SRV_DOMAIN"
[ -x "/usr/sbin/llxcfg-showvars" ] && eval `/usr/sbin/llxcfg-showvars ${NET_VAR_LIST}`

# si se establece un valor para SRV_NIC, se utilizará dicho valor
# si no, se buscará el establecido para el dhcp y si no el DEFAULT

DEFAULT_SRV_NIC="eth0"

[ -z "$SRV_DOMAIN" ] && SRV_DOMAIN=$DEFAULT_SRV_DOMAIN
[ -z "$DEFAULT_SRV_IP" ] && DEFAULT_SRV_IP="10.0.2.254"
[ -z "$DEFAULT_SRV_MASK" ] && DEFAULT_SRV_MASK="255.255.255.0"

# BEGIN GENERIC NETWORK DETECTION FUNCTIONS

netcmd_wrapper(){
   local OLDLANG
   OLDLANG="$LANG"
   OLDLANGUAGE="$LANGUAGE"
   LANG="C"
   LANGUAGE="C"
   export TERM="LEGACY"
   ${@}
   LANG="$OLDLANG"
   LANGUAGE="$OLDLANGUAGE"
}

check_ETH_DEVICE(){
   grep -q "^[[:space:]]*${1}:" /proc/net/dev || return 1
   return 0
}

check_ETH_DEVICE_EXTENDED(){
   $($IFCONFIG -a 2>/dev/null |grep -q "^$1[[:blank:]]\+") || return 1
   return 0
}

get_def_route() {
   DEF_ROUTE=`${ROUTE} -n |grep "^0.0.0.0" |awk '{print $2}'`
   echo "$DEF_ROUTE"
}

get_mac() {
   check_ETH_DEVICE "$1" || return 0
   ETH_MAC=`${IFCONFIG} $1 | sed -ne "s%^.*HWaddr %%p"`
   echo "$ETH_MAC"
}

ip_list(){
   ${IFCONFIG} "$1" | grep "inet addr:" | cut -d ":" -f 2 | awk '{print $1}'
   return 0
}

is_current_ip(){
   [ "$1" ] || return 1
   ip_list |grep -qFx "$1" || return 1
   return 0
}

get_current_ip() {
   check_ETH_DEVICE_EXTENDED "$1" || return 0
   ETH_IP=`${IFCONFIG} "$1" | grep "inet addr:" | cut -d ":" -f 2 | awk '{print $1}'`
   echo "$ETH_IP"
}

get_current_bcast() {
   check_ETH_DEVICE_EXTENDED "$1" || return 0
   ETH_IP=`${IFCONFIG} "$1" | grep "inet addr:" | cut -d ":" -f 3 | awk '{print $1}'`
   echo "$ETH_IP"
}

get_current_netmask() {
   check_ETH_DEVICE_EXTENDED "$1" || return 0
   ETH_IP=`${IFCONFIG} "$1" | grep "inet addr:" | cut -d ":" -f 4 | awk '{print $1}'`
   echo "$ETH_IP"
}

get_current_bitmask() {
   check_ETH_DEVICE_EXTENDED "$1" || return 0
   ip_ad="$(get_current_ip "$1")"
   ip_nm="$(get_current_netmask "$1")"
   ipcalc_bitmask "$ip_ad $ip_nm"
}

is_ethernet_card() {
   check_ETH_DEVICE "$1" || return 1
   if (${IFCONFIG} "$1" 2> /dev/null |grep -q "Link encap:Ethernet"); then
      # es ethernet (incl. wlan) y no firewire u otras cosas
      return 0
   fi
   return 1
}

is_wireless_card() {
   check_ETH_DEVICE "$1" || return 1
   if [ -e /proc/net/wireless ]; then
      if ( grep -q "^ *${1}:" /proc/net/wireless ); then
         return 0
      fi
   fi
   return 1
}

get_alias() {
   ALIAS_LIST="$(${IFCONFIG}|sed -rn  /^[[:alnum:]]+[[:digit:]]\:[[:digit:]]+/s/[[:blank:]].*//p)"
   echo "${ALIAS_LIST}"
}

get_network_cards() {
   # argumento (opcional): wireless
   if [ "$1" = "wireless" ]; then
      ETH_LIST0="$(cat /proc/net/wireless | awk -F: '/eth.:|tr.:|wlan.:|ra.:/{print $1}')"
   else
      ETH_LIST0="$(cat /proc/net/dev | awk -F: '/eth.:|tr.:|wlan.:|ra.:/{print $1}')"
   fi

   ETH_LIST=""
   if [ -n "${ETH_LIST0}" ]; then
      for NET_CARD in ${ETH_LIST0}; do
         if is_ethernet_card ${NET_CARD}; then
	    ETH_LIST="${ETH_LIST} ${NET_CARD}"
	 fi
      done
   fi
   if [ "$1" = "alias" ]; then
      ETH_LIST="$ETH_LIST $(get_alias)"
   fi
   echo $ETH_LIST
}

count_network_cards() {
   get_network_cards |wc -w
}

get_wireless_cards() {
   WIFI_LIST=""
   if [ -e /proc/net/wireless ]; then
      WIFI_LIST="`get_network_cards wireless`"
   fi
   echo ${WIFI_LIST}
}

get_nic_status() {
   check_ETH_DEVICE "$1" || return 0
   ETH_STATUS="DOWN"
   if ( ${IFCONFIG} "$1" 2> /dev/null |grep -q "^ *UP" ); then 
      ETH_STATUS="UP"
   fi
  echo $ETH_STATUS
}

get_first_nic() {
   ETH_LIST=`get_network_cards`
   for i in ${ETH_LIST}; do
      if [ "`get_nic_status $i`" == "UP" ]; then
	 echo $i
         return 0
      fi
   done
   echo ""
   return 1
}

check_ipcalc_arg() {
   # Las siguientes funciones, admiten como argumentos
   # una dirección IP y una máscara de subred, o bien en la forma:
   #      IP_ADDR NET_MASK
   # o bien como:
   #      IP_ADDR/NET_MASK
   # esta función verifica que se está utilizando alguna de las dos

   IPCALC_ARG="$@"
   # Comprobamos argumentos:
   if ! (echo "$IPCALC_ARG" |egrep -q "^ *(([[:digit:]]{1,3}\.){3}[[:digit:]]{1,3} *){2} *$" ); then
      #no es de la primera forma:  IP_ADDR NET_MASK
      if ! ( echo "$IPCALC_ARG" |egrep -q "^ *([[:digit:]]{1,3}\.){3}[[:digit:]]{1,3}\/[[:digit:]]{1,2} *$" ); then
         # ni de la segunda forma: IP_ADDR/NET_MASK
	 return 1
      fi
   fi
   return 0
}

ipcalc_value() {
   # argumentos:
   #   cadena_a_buscar , direccioón ip, máscara de subred
   #       (en forma IP_ADDR NET_MASK o IP_ADDR/NET_MASK)
   # 
   # Recupera el valor correspondiente a "cadena_a_buscar"
   # de la salida del comando ipcalc
   # se utiliza para calcular: HostMin, HostMax,  Broadcast, Network, Bitmask
   RETURN_VALUE=""
   SEARCH_STRING="$1"
   IPCALC_ARG="$2 $3"
   if [ -n "$SEARCH_STRING" ]; then
      if check_ipcalc_arg "$IPCALC_ARG"; then
         RETURN_VALUE="`$IPCALC -n $IPCALC_ARG | grep \"${SEARCH_STRING}:\" | cut -d \":\" -f 2 | awk '{print $1}'`"
	 echo "$RETURN_VALUE"
	 return 0
      fi
   fi
   echo ""
   return 1
}

ipcalc_bitaddress(){
   # argumentos: IP_ADDR NET_MASK o IP_ADDR/NET_MASK
   #
   # Devuelve la dirección IP que se pasa como argumento
   # en forma de número binario de 32 bits
   #
   IPCALC_ARG="$1 $2"
   RETURN_VALUE=""
   if check_ipcalc_arg "$IPCALC_ARG"; then
      SEARCH_STRING="Address:"
      BITS="`$IPCALC -n $IPCALC_ARG |sed -ne /^${SEARCH_STRING}/s%^${SEARCH_STRING}[[:space:]]*[^[:space:]]*[[:space:]]*%%p |cut -d . -f 1- --output-delimiter=\ `"
      for b in ${BITS}; do
         RETURN_VALUE="${RETURN_VALUE}`echo ${b}`"
      done
   fi
   echo "${RETURN_VALUE}"
}

ip2num(){
   # argumentos: IP_ADDR NET_MASK o IP_ADDR/NET_MASK
   #
   # Devuelve el valor numérico (en decimal) equivalente a la dirección IP
   # que se pasa como argumento considerada como un número binario de 32 bits
   #
   echo $((2#`ipcalc_bitaddress $@`))
}

ipcalc_addnum2ip(){
   # argumentos: NUM_TO_ADD IP_ADDR NET_MASK | IP_ADDR/NET_MASK
   #
   # Devuelve la dirección IP resultante tras "añadir" el valor numérico que se
   # pasa como primer argumento a la dirección suministrada como segundo argumento.
   # No se comprueba el signo de NUM_TO_ADD, por lo que también es posible usar la
   # función para "restar" direcciones usando un valor negativo.
   # Si el resultado no se encuentra dentro de los límites válidos para la red correspondiente,
   # se devuelve el valor máximo (hostmax) o el mínimo (hostmin) según corresponda.
   #
   NUM_TO_ADD=$(($1))
   shift
   CURRENT_IP_NUM=`ip2num $@`
   IP_MASK=`ipcalc_netmask $@`
   MIN_IP=`ipcalc_hostmin $@`
   MIN_IP_NUM=`ip2num ${MIN_IP} ${IP_MASK}`
   MAX_IP=`ipcalc_hostmax $@`
   MAX_IP_NUM=`ip2num ${MAX_IP} ${IP_MASK}`
   NEXT_IP_NUM=$((${CURRENT_IP_NUM}+${NUM_TO_ADD}))
   
   [ ${NEXT_IP_NUM} -lt ${MIN_IP_NUM} ] && NEXT_IP_NUM=${MIN_IP_NUM}
   [ ${NEXT_IP_NUM} -gt ${MAX_IP_NUM} ] && NEXT_IP_NUM=${MAX_IP_NUM}
   num2ip ${NEXT_IP_NUM}
}

num2ip(){
   NEXT_IP_NUM=$(($1))
   NEXT_IP="$((${NEXT_IP_NUM}%256))"
   BYTE_COUNT=0
   while [ ${BYTE_COUNT} -le 2 ]; do
      NEXT_IP_NUM=$((${NEXT_IP_NUM}>>8))
      NEXT_IP="$((${NEXT_IP_NUM}%256)).${NEXT_IP}"
      BYTE_COUNT=$((${BYTE_COUNT}+1))
   done
   echo "${NEXT_IP}"
}

ipcalc_broadcast() {
   # argumentos: IP_ADDR NET_MASK o IP_ADDR/NET_MASK
   ipcalc_value "Broadcast" "$@"
}

ipcalc_hostmin() {
   # argumentos: IP_ADDR NET_MASK o IP_ADDR/NET_MASK
   ipcalc_value "HostMin" "$@"
}

ipcalc_hostmax() {
   # argumentos: IP_ADDR NET_MASK o IP_ADDR/NET_MASK
   ipcalc_value "HostMax" "$@"
}

ipcalc_netmask() {
   # argumentos: IP_ADDR NET_MASK o IP_ADDR/NET_MASK
   ipcalc_value "Netmask" "$@"
}

ipcalc_network() {
   # argumentos: IP_ADDR NET_MASK o IP_ADDR/NET_MASK
   NET_VALUE="`ipcalc_value Network $@`"
   NET_ADDR=""
   if [ -n "$NET_VALUE" ]; then
      NET_ADDR="`echo \"$NET_VALUE\" | cut -d \"/\" -f 1 | awk '{print $1}'`"
   fi
   echo "$NET_ADDR"
}

ipcalc_bitmask() {
   # argumentos: IP_ADDR NET_MASK o IP_ADDR/NET_MASK
   NET_VALUE="`ipcalc_value Network $@`"
   BIT_MASK=""
   if [ -n "$NET_VALUE" ]; then
      BIT_MASK="`echo \"$NET_VALUE\" | cut -d \"/\" -f 2 | awk '{print $1}'`"
   fi
   echo "$BIT_MASK"
}

ip_reverse(){
   # argumentos: IP_ADDR [NUM_BYTES]
   #
   # Invierte el orden de los bytes de la dirección suministrada y devuelve
   # la dirección inversa "parcial" formada por el número de bytes indicado
   # por NUM_BYTES (por defecto 4, la dirección completa)
   declare -a DIR_IP
   local NUM_BYTES i REVERSE_IP
   DIR_IP=($(echo "$1" |tr "." " "))
   NUM_BYTES=$2
   [ -z "$NUM_BYTES" ] && NUM_BYTES=4
   REVERSE_IP=""
   i=0
   while [ $i -le $NUM_BYTES ] ; do
      REVERSE_IP="${REVERSE_IP} ${DIR_IP[$((4 - $i))]}"
      i=$(($i + 1))
   done
   echo $REVERSE_IP |tr " " "."
}

split_network(){
   # argumentos: IP_ADDR NET_MASK | IP_ADDR/NET_MASK
   #
   # Divide la red correspondiente a la dirección suministrada en dos subredes
   # Devuelve la direción IP del primer host de la "otra semired", es decir, 
   # de la "mitad" en la que NO se encuentra la la dirección que se ha pasado como
   # argumento. El valor devuelto está en la forma: IP/NET_MASK
   #
   CURRENT_IP="$@"
   OLD_NET="`ipcalc_network ${CURRENT_IP}`"
   OLD_BITS="`ipcalc_bitmask ${CURRENT_IP}`"
   OLD_BCAST="`ipcalc_broadcast ${CURRENT_IP}`"
   OLD_IP="`ipcalc_value Address ${CURRENT_IP}`"
   #calculamos la dirección de red aumentando 1 bit
   # la mascara (para dividir el rango en 2)
   NEW_BITS=$((${OLD_BITS}+1))
   NEW_NET="`ipcalc_network ${OLD_IP}/${NEW_BITS}`"
   #Obtenemos el primer host válido de la nueva subred
   if [ "${OLD_NET}" = "${NEW_NET}" ]; then
      NEW_FIRST="`ipcalc_hostmin ${OLD_BCAST}/${NEW_BITS}`"
   else
      NEW_FIRST="`ipcalc_hostmin ${OLD_IP}/${OLD_BITS}`"
   fi
   echo "${NEW_FIRST}/${NEW_BITS}"
}

get_mac_from_ip(){
   IP_ADDR="$1"
   if [ "${IP_ADDR}" ]; then
      if ${PING} ${IP_ADDR} >/dev/null ;then
         ARP_LINE=`${ARP} |grep "^${IP_ADDR}"`
         echo ${ARP_LINE} |cut -f 3 -d \ 
      fi
   fi
}   

is_remote_address(){
   [ "$(get_mac_from_ip "$1")" = "" ] || return 0
   return 1
}

get_nic_driver() {
   check_ETH_DEVICE "$1" || return 0
   if [ "$1" ]; then
      DRIVER_DIR="/sys/class/net/${1}/driver"
      # path depends of kernel version ...
      [ -e ${DRIVER_DIR} ] || DRIVER_DIR="/sys/class/net/${1}/device/driver"
      [ -e ${DRIVER_DIR} ] && basename `readlink -f ${DRIVER_DIR}`
   fi
}

get_nic_description() {
   ETH_DEVICE="$1"
   check_ETH_DEVICE "$ETH_DEVICE" || return 0
   if [ "${ETH_DEVICE}" ]; then
      DEVICE_DIR0="/sys/class/net/${ETH_DEVICE}/device"
      if [ -e ${DEVICE_DIR0} ]; then
         DEVICE_DIR="`readlink -f ${DEVICE_DIR0}`"
	 DEVICE_NUMBER="`basename ${DEVICE_DIR}`"
	 # deteccion bus ...
	 if echo "${DEVICE_DIR}" |grep -q "/usb"; then
	    # special case, usb network device
	    DEV_DES="(USB)"
	    BUS_NUM=${DEVICE_NUMBER%%\-*}
	    DEV_FILE="${DEVICE_DIR}/../devnum"
	    if [ -e ${DEV_FILE} ]; then
	       DEV_NUM=`cat ${DEV_FILE}`
	       DEV_DES="${DEV_DES} `${LSUSB} -vs ${BUS_NUM}:${DEV_NUM} |sed -ne "s%[[:space:]]*idProduct[[:space:]]*\(0x[^[:space:]]*\)*[[:space:]]*\(.*\)$%\2%p"`"
	    fi
	    echo ${DEV_DES}
	 else
	    ${LSPCI} | sed -ne "s%^${DEVICE_NUMBER}[[:space:]]*\(.*\)$%\1%p" 2>/dev/null
	 fi
      fi
   fi
}

get_link_status() {
   ETH_DEVICE="$1"
   check_ETH_DEVICE "${ETH_DEVICE}" || return 0
   [ `id -u` -ne 0 ] && return 0
   CURRENT_STATUS=`get_nic_status ${ETH_DEVICE}`
   if [ "${CURRENT_STATUS}" = "DOWN" ] ; then
      ${IFCONFIG} ${ETH_DEVICE} up
      sleep 0.5
   fi
   LINK_STATUS="DOWN"
   if ${ETHTOOL} ${ETH_DEVICE} |grep -q "^[[:space:]]*Link detected: yes$" ; then
      LINK_STATUS="UP"
   fi
   [ "${CURRENT_STATUS}" = "DOWN" ] && ${IFCONFIG} ${ETH_DEVICE} down
   echo $LINK_STATUS
}


# END GENERIC NETWORK DETECTION FUNCTIONS

# BEGIN LLIUREX SPECIFIC NETWORK FUNCTIONS

get_srv_nic() {
   SRV_ETH="$SRV_NIC"
   # buscamos interfaz de dhcp
   if [ -z "${SRV_ETH}" ]; then
      if [ -f /etc/default/dhcp3-server ]; then
         . /etc/default/dhcp3-server
         SRV_ETH=${INTERFACES%% *}
      fi
   fi
   [ -z "${SRV_ETH}" ] &&  SRV_ETH="$DEFAULT_SRV_NIC"
   echo "$SRV_ETH"
}

get_srv_ip() {
   SRV_ETH="`get_srv_nic`"
   SRV_IP=""
   if [ "$1" != "default" -a  "$SRV_ETH" != "" ]; then 
      SRV_IP=`get_current_ip $SRV_ETH`
   fi
   if [ -z "$SRV_IP" ]; then
      # LliureX default
      SRV_IP=$DEFAULT_SRV_IP
   fi
   echo "$SRV_IP"
}

get_srv_mask() {
   SRV_ETH="`get_srv_nic`"
   SRV_MASK=""
   if [ "$1" != "default" -a "$SRV_ETH" != "" ]; then 
      SRV_MASK=`get_current_netmask $SRV_ETH`
   fi
   if [ -z "$SRV_MASK" ]; then
      # LliureX default
      SRV_MASK=$DEFAULT_SRV_MASK
   fi
   echo "$SRV_MASK"
}

get_srv_bitmask(){
   ipcalc_bitmask `get_srv_ip` `get_srv_mask`
}

get_srv_bcast() {
   SRV_ETH="`get_srv_nic`"
   SRV_BCAST=""
   if [ "$1" != "default" -a "$SRV_ETH" != "" ]; then 
      SRV_BCAST=`get_current_bcast $SRV_ETH`
   fi
   if [ -z "$SRV_BCAST" ]; then
      # LliureX default
      SRV_BCAST="`ipcalc_broadcast $DEFAULT_SRV_IP $DEFAULT_SRV_MASK`"

   fi
   echo "$SRV_BCAST"
}

get_srv_network() {
   if [ "$1" = "default" ]; then 
      SRV_IP=$DEFAULT_SRV_IP
      SRV_MASK=$DEFAULT_SRV_MASK
   else
      SRV_IP="`get_srv_ip`"
      SRV_MASK="`get_srv_mask`"
   fi
   SRV_NET="`ipcalc_network $SRV_IP $SRV_MASK`"
   echo "$SRV_NET"
}

get_srv_domain() {
   if [ "$1" = "default" ]; then 
      echo "$DEFAULT_SRV_DOMAIN"
   else
      echo "$SRV_DOMAIN"
   fi
}

get_srv_reverse_domain() {
   local n NBITS NBYTES
   declare -a IP_ARRAY
   NBITS=$(get_srv_bitmask)
   NBYTES=$(($NBITS/8))
   IP_ARRAY=($(echo "$(get_srv_ip)" |tr "." " "))
   n=1
   while [ $n -le $NBYTES ] ; do
      echo -n "${IP_ARRAY[$(($NBYTES - $n))]}."
      n=$(($n + 1))
   done
   echo "in-addr.arpa"
}

get_srv_mac() {
   SRV_ETH="`get_srv_nic`"
   SRV_MAC=`get_mac $SRV_ETH`
   echo "$SRV_MAC"
}

get_srv_name(){
   echo "server"
}

get_noreg_host_name(){
   # argumentos: HOST_IP
   # Devuelve el nombre de host correspondiente a la IP dinámica suministrada
   # Los nombres se construyen concatenando el prefijo "UNREG_HOSTNAME_PREFIX" y
   # la dirección IP (con los puntos cambiados por "-")
   #
   IP_TXT="`echo $1 |cut -f 1- -d . --output-delimiter=-`"
   echo "${UNREG_HOSTNAME_PREFIX}${IP_TXT}"
}

is_valid_ip() {
   # Las siguientes funciones, admiten como argumentos
   # una dirección IP y una máscara de subred, o bien en la forma:
   #      IP_ADDR NET_MASK
   # o bien como:
   #      IP_ADDR/NET_MASK
   # esta función verifica que se está utilizando alguna de las dos

   IPCALC_ARG="$@"
   # Comprobamos argumentos:
	if (echo "$IPCALC_ARG" |egrep -q "^ *(([[:digit:]]{1,3}\.){3}[[:digit:]]{1,3} *){2} *$"); then
		FPARAMETER=$(echo $1 | cut -d "." -f1) 
		if [  $FPARAMETER -lt 224 -a $FPARAMETER -gt 0 ]; then

			SPARAMETER=$(echo $1 | cut -d "." -f2)
			TPARAMETER=$(echo $1 | cut -d "." -f3)
			FHPARAMETER=$(echo $1 | cut -d "." -f4)
			if [ $SPARAMETER -lt 255 -a $TPARAMETER -lt 255 -a $FHPARAMETER -lt 255 ] ; then
			
				return 0

			fi
		fi	
	fi
   return 1
}



# END LLIUREX SPECIFIC NETWORK FUNCTIONS


